@@ -12,12 +12,12 @@
 #=======================================================================
 package PDF::API2::Basic::PDF::Null;
 
-our $VERSION = '2.025'; # VERSION
-
 use base 'PDF::API2::Basic::PDF::Objind';
 
 use strict;
 
+our $VERSION = '2.038'; # VERSION
+
 =head1 NAME
 
 PDF::API2::Basic::PDF::Null - PDF Null type object.  This is a subclass of
@@ -88,7 +88,7 @@ Return undef.
 =cut
 
 sub val {
-    return undef;
+    return undef; ## no critic (undef is intentional)
 }
 
 1;
