@@ -5,7 +5,7 @@ use base 'PDF::API2::Basic::PDF::Dict';
 use strict;
 no warnings qw[ recursion uninitialized ];
 
-our $VERSION = '2.030'; # VERSION
+our $VERSION = '2.038'; # VERSION
 
 use Encode qw(:all);
 
@@ -37,23 +37,13 @@ sub new
     return($self);
 }
 
-=item $dest = PDF::API2::NamedDestination->new_api $api
-
-Returns a destination object. This method is different from 'new' that
-it needs an PDF::API2-object rather than a PDF::API2::PDF::File-object.
-
-=cut
-
+# Deprecated (warning added in 2.031)
 sub new_api {
-  my ($class,$api,@opts)=@_;
+    my ($class, $api2, @options) = @_;
+    warnings::warnif('deprecated', q{Call to deprecated method "new_api($api2, ...)".  Replace with "new($api2->{'pdf'}, ...)"});
 
-  my $obj=$class->new($api->{pdf},@opts);
-
-  $api->{pdf}->new_obj($obj) unless($obj->is_obj($api->{pdf}));
-
-  $api->{pdf}->out_obj($api->{pages});
-
-  return($obj);
+    my $destination = $class->new($api2->{'pdf'}, @options);
+    return $destination;
 }
 
 =item $dest->link $page, %opts
@@ -85,20 +75,8 @@ sub url
     my ($self,$url,%opts)=@_;
 
     $self->{S}=PDFName('URI');
-    if(is_utf8($url))
-    {
-        # URI must be 7-bit ascii
-        utf8::downgrade($url);
-    }
     $self->{URI}=PDFStr($url);
 
-    # this will come again -- since the utf8 urls are coming !
-    # -- fredo
-    #if(is_utf8($url) || utf8::valid($url)) {
-    #    $self->{URI}=PDFUtf($url);
-    #} else {
-    #    $self->{URI}=PDFStr($url);
-    #}
     return($self);
 }
 
@@ -114,20 +92,8 @@ sub file
     my ($self,$url,%opts)=@_;
 
     $self->{S}=PDFName('Launch');
-    if(is_utf8($url))
-    {
-        # URI must be 7-bit ascii
-        utf8::downgrade($url);
-    }
     $self->{F}=PDFStr($url);
 
-    # this will come again -- since the utf8 urls are coming !
-    # -- fredo
-    #if(is_utf8($url) || utf8::valid($url)) {
-    #    $self->{F}=PDFUtf($url);
-    #} else {
-    #    $self->{F}=PDFStr($url);
-    #}
     return($self);
 }
 
@@ -143,21 +109,8 @@ sub pdfile
     my ($self,$url,$pnum,%opts)=@_;
 
     $self->{S}=PDFName('GoToR');
-    if(is_utf8($url))
-    {
-        # URI must be 7-bit ascii
-        utf8::downgrade($url);
-    }
     $self->{F}=PDFStr($url);
 
-    # this will come again -- since the utf8 urls are coming !
-    # -- fredo
-    #if(is_utf8($url) || utf8::valid($url)) {
-    #    $self->{F}=PDFUtf($url);
-    #} else {
-    #    $self->{F}=PDFStr($url);
-    #}
-
     $self->dest(PDFNum($pnum),%opts);
 
     return($self);
