@@ -1,8 +1,11 @@
 package PDF::API2::Resource::Font::CoreFont::helveticaboldoblique;
 
-our $VERSION = '2.020'; # VERSION
+use strict;
+use warnings;
 
-$FONTDATA = {
+our $VERSION = '2.038'; # VERSION
+
+sub data { return {
     'fontname' => 'Helvetica-BoldOblique',
     'type' => 'Type1',
     'apiname' => 'HeBi',
@@ -509,4 +512,6 @@ $FONTDATA = {
         'fi' => '611',                           # U+0xFB01
         'fl' => '611',                           # U+0xFB02
     }, # HORIZ. WIDTH TABLE
-};
+} };
+
+1;
